﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ImageVisualizer.Zoom;

namespace ImageVisualizer.GUI
{
    /// <summary>
    /// Interaction logic for ImageDisplay.xaml
    /// </summary>
    public partial class ImageDisplay : UserControl
    {
        #region Fields

        System.Drawing.Bitmap _sourceBmp;

        #endregion

        #region Constructors

        public ImageDisplay(System.Drawing.Bitmap sourceBitmap)
            : this()
        {
            _sourceBmp=sourceBitmap;
            DisplayImage.Source = ImageConverters.ConvertBitmapToBitmapSource(sourceBitmap);
        }

        private ImageDisplay()
        {
            InitializeComponent();
        }

        #endregion

        #region Properties

        internal double Zoom
        {
            get { return (double)GetValue(ZoomProperty); }
            set { SetValue(ZoomProperty, value); }
        }

        #endregion

        #region Methods

        bool IsCtrlPressed()
        {
            bool IsLeftCtrlDown = (Keyboard.GetKeyStates(Key.LeftCtrl) == KeyStates.Down);
            bool IsRightCtrlDown = (Keyboard.GetKeyStates(Key.RightCtrl) == KeyStates.Down);
            
            return IsLeftCtrlDown || IsRightCtrlDown;
        }

        void ChangeZoomLevel(bool increaseZoom)
        {
            double newZoomFactor = Zoom + (increaseZoom ? 0.1 : -0.1);
            try
            {
                Zoom = newZoomFactor;
            }
            catch (ArgumentException)
            {
                //Value was out of range. Just don't zoom
            }
        }

        private void UpdateInfoLabel()
        {
            Point mousePositionInImage = Mouse.GetPosition(DisplayImage);

            int mouseX = (int)mousePositionInImage.X;
            int mouseY = (int)mousePositionInImage.Y;

            System.Drawing.Color pixelColor = _sourceBmp.GetPixel(mouseX, mouseY);
            string hexColor = System.Drawing.ColorTranslator.ToHtml(pixelColor);

            UpdateInfoLabel((int)mousePositionInImage.X,
                            (int)mousePositionInImage.Y,
                            hexColor);
        }

        private void UpdateInfoLabel(int mouseX, int mouseY, string colorValue)
        {
            const string PixelInfoText = "({0}x{1}) {2}";

            PixelInfo.Content = string.Format(PixelInfoText, mouseX, mouseY, colorValue);
        }

        #endregion

        #region Event Handlers

        private static bool ZoomChanged(object value)
        {
            return ZoomConverter.IsZoomValueValid((double)value);
        }

        private void DisplayImage_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            if (IsCtrlPressed())
            {
                bool increaseZoom = (e.Delta>=0);
                ChangeZoomLevel(increaseZoom);
                e.Handled = true;
            }
        }

        private void DisplayImage_MouseMove(object sender, MouseEventArgs e)
        {
            UpdateInfoLabel();
        }
 
        #endregion

        #region Dependency Properties

        /// <summary>
        /// Dependency property for image zoom
        /// </summary>
        /// <remarks>Pay attention that the default value is 1.0 (double) and not 1 (int)</remarks>
        private static readonly DependencyProperty ZoomProperty = DependencyProperty.Register("Zoom", typeof(double), typeof(ImageDisplay),
            new FrameworkPropertyMetadata(1.0, FrameworkPropertyMetadataOptions.AffectsMeasure), new ValidateValueCallback(ZoomChanged));

        #endregion     
    }
}
