//Copyright (c) Victor Hurdugaci
//http://victorhurdugaci.com
//This code is licensed under the Non-Profit Open Software License version 3.0
//Full license text: http://www.opensource.org/licenses/NOSL3.0.html

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Data;

namespace ImageVisualizer.Zoom
{
    /// <summary>
    /// Converts a percent (string) to a zoom value (double)
    /// </summary>
    class ZoomConverter:IValueConverter
    {
        #region Constants

        private static readonly int PercentDenominator = 100;
        
        private static readonly double ZoomMinimumValue = 0.1;
        private static readonly double ZoomMaximumValue = 16;

        #endregion

        #region Methods

        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            //Number with no precision followed by "%"
            const string ZoomFactorTextFormat = "{0:0}%";

            double scaleFactor = (double)value;

            return string.Format(ZoomFactorTextFormat, scaleFactor * PercentDenominator);
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            string strValue = value as string;

            strValue = strValue.TrimEnd('%');
            double percentValue = double.Parse(strValue);

            double scaleFactor = percentValue / PercentDenominator;

            return scaleFactor;
        }

        public static bool IsZoomValueValid(double value)
        {
            return (ZoomMinimumValue <= value) && (value <= ZoomMaximumValue);
        }

        #endregion
    }
}
